#!/bin/bash

# getXuser: get the user for the specified X display, or if none is
# specified, then the user for whatever X display we find.
# input: $displaynum: the X display to query
# output: $XAUTHORITY: the path to the xauth data used for connecting to the
#                      detected X display.
#         $XUSER: the username of the user who owns the display.
getXuser() {
	local plist display uid user startx pid userhome IFS

	if [ "$displaynum" ]; then
		display=":$displaynum"
	else
		display=.+
	fi

	user=

#alex: use systemd/logind
  if [ -x /bin/loginctl ]; then
      sess=$(loginctl --no-legend |awk '{print $1}')
      for s in $sess; do
        dpy=$(loginctl show-session --property=Display "$s" | cut -f2 -d'=')
        if [ ":$displaynum" = "$dpy" ]; then
          user=$(loginctl show-session --property=Name "$s" | cut -f2 -d'=')
          if [ "$user" != "fly-dm" ]; then
            break
          fi
        fi
      done
  fi

	if [ -z "$user" ]; then
		plist=$(pinky -fw) || pwf_error "pinky lost"
		while read l; do
			set -- $l
			eval lastpp=\$$#
			for ds in $2 $lastpp; do
				case $ds in
					$display)
						user=$1
						break
						;;
				esac
			done
			[ -z "$user" ] || break
			for ds in $2 $lastpp; do
				case $ds in
					$display.0)
						user=$1
						break
						;;
				esac
			done
			[ -z "$user" ] || break
		done <<-EOF
			$plist
		EOF
	fi

	if [ -z "$user" ]; then
		startx=$(pgrep -n startx || :)
		[ -z "$startx" ] || user=$(ps -o user --no-headers $startx || :)
	fi

	if [ x"$user" != x ]; then
        	for pid in `ps -U "$user" -o pid=`; do
			if grep -z -q XAUTHORITY /proc/$pid/environ; then
				XAUTHORITY="$(grep -z XAUTHORITY /proc/$pid/environ | cut -d= -f2-)"
				export XAUTHORITY
				break
			fi
		done
		if [ -z "$XAUTHORITY" ]; then
			userhome="$(getent passwd "$user" | cut -d: -f6)"
			XAUTHORITY="$userhome/.Xauthority"
			export XAUTHORITY
		fi
	else
		export XAUTHORITY=
	fi
	XUSER="$user"
	export XUSER
}

if [ -z "$1" ]; then
echo "Use fly-send-msg.sh [mesage | command of fly-wmfunc] to send messge or command to all users"
exit
fi

d=/tmp/.X11-unix
for x in $d/X*; do
    displaynum=${x#$d/X}
    getXuser;
    if [ "x$XAUTHORITY" != x ]; then
        	export DISPLAY=:"$displaynum"

        	case $1 in
        	  FLYWM_*)
			  su "$XUSER" -s /bin/sh -c "/usr/bin/fly-wmfunc $1"
			  #for other MAC levels to be tested
              #for SE1.5
			  /usr/bin/find /var/private/tmp -type p -name "fly-wm-*" -exec /bin/sh -c 'echo $1 > {}' \;
			  #for SE1.6
			  /usr/bin/find /var/run/user -type p -name "fly-wm-*" -exec /bin/sh -c 'echo $1 > {}' \;
              ;;
              *)
        	  su "$XUSER" -s /bin/sh -c "/usr/bin/fly-dialog --msgbox '$1' &"
              ;;
            esac
	fi
done
